{# -*- coding: utf-8 -*-

  This file is part of Invenio.
  Copyright (C) 2015-2018 CERN.

  Invenio is free software; you can redistribute it and/or modify it
  under the terms of the MIT License; see LICENSE file for more details.
#}

{%- extends config.OAUTH2SERVER_COVER_TEMPLATE %}

{%- import "invenio_oauth2server/settings/helpers.html" as helpers with context %}

{%- block page_body %}
<div class="row">
  <div class="col-md-8 col-md-offset-2">
  {{ helpers.panel_start(
    _('Invalid authorization request'),
    icon='fa fa-warning fa-fw'
  ) }}

  {{ _('Invalid authorization request') }}

  {{ _('The service that redirected your here made an invalid authorization request (error code: %(x_error)s).',
       x_error=error.error) }}

  <div class="row">
    <hr />
    <div align="center">
      <a href="/" class="btn btn-info">
        {{ _('Get me out of here!') }}
      </a>
    </div>
    {{ helpers.panel_end() }}
  </div>
</div>
{%- endblock %}
